@@ -27,7 +27,7 @@ from pyhilo.const import (
     API_NOTIFICATIONS_ENDPOINT,
     API_REGISTRATION_ENDPOINT,
     API_REGISTRATION_HEADERS,
-    AUTOMATION_DEVICEHUB_ENDPOINT,
+    AUTOMATION_CHALLENGE_ENDPOINT,
     DEFAULT_STATE_FILE,
     DEFAULT_USER_AGENT,
     FB_APP_ID,
@@ -51,7 +51,7 @@ from pyhilo.util.state import (
     get_state,
     set_state,
 )
-from pyhilo.websocket import WebsocketClient
+from pyhilo.websocket import WebsocketClient, WebsocketManager
 
 
 class API:
@@ -81,9 +81,17 @@ class API:
         self.device_attributes = get_device_attributes()
         self.session: ClientSession = session
         self._oauth_session = oauth_session
+        self.websocket_devices: WebsocketClient
+        # Backward compatibility during transition to websocket for challenges. Currently the HA Hilo integration
+        # uses the .websocket attribute. Re-added this attribute and point to the same object as websocket_devices.
+        # Should be removed once the transition to the challenge websocket is completed everywhere.
         self.websocket: WebsocketClient
+        self.websocket_challenges: WebsocketClient
         self.log_traces = log_traces
         self._get_device_callbacks: list[Callable[..., Any]] = []
+        self.ws_url: str = ""
+        self.ws_token: str = ""
+        self.endpoint: str = ""
 
     @classmethod
     async def async_create(
@@ -132,6 +140,9 @@ class API:
         if not self._oauth_session.valid_token:
             await self._oauth_session.async_ensure_token_valid()
 
+        access_token = str(self._oauth_session.token["access_token"])
+        LOG.debug(f"ic-dev21 access token is {access_token}")
+
         return str(self._oauth_session.token["access_token"])
 
     def dev_atts(
@@ -216,6 +227,8 @@ class API:
         :rtype: dict[str, Any]
         """
         kwargs.setdefault("headers", self.headers)
+        access_token = await self.async_get_access_token()
+
         if endpoint.startswith(API_REGISTRATION_ENDPOINT):
             kwargs["headers"] = {**kwargs["headers"], **API_REGISTRATION_HEADERS}
         if endpoint.startswith(FB_INSTALL_ENDPOINT):
@@ -223,10 +236,15 @@ class API:
         if endpoint.startswith(ANDROID_CLIENT_ENDPOINT):
             kwargs["headers"] = {**kwargs["headers"], **ANDROID_CLIENT_HEADERS}
         if host == API_HOSTNAME:
-            access_token = await self.async_get_access_token()
             kwargs["headers"]["authorization"] = f"Bearer {access_token}"
         kwargs["headers"]["Host"] = host
 
+        # ic-dev21 trying Leicas suggestion
+        if endpoint.startswith(AUTOMATION_CHALLENGE_ENDPOINT):
+            # remove Ocp-Apim-Subscription-Key header to avoid 401 error
+            kwargs["headers"].pop("Ocp-Apim-Subscription-Key", None)
+            kwargs["headers"]["authorization"] = f"Bearer {access_token}"
+
         data: dict[str, Any] = {}
         url = parse.urljoin(f"https://{host}", endpoint)
         if self.log_traces:
@@ -303,8 +321,9 @@ class API:
                 LOG.info(
                     "401 detected on websocket, refreshing websocket token. Old url: {self.ws_url} Old Token: {self.ws_token}"
                 )
+                LOG.info(f"401 detected on {err.request_info.url}")
                 async with self._backoff_refresh_lock_ws:
-                    (self.ws_url, self.ws_token) = await self.post_devicehub_negociate()
+                    await self.refresh_ws_token()
                     await self.get_websocket_params()
                 return
 
@@ -354,33 +373,31 @@ class API:
         LOG.debug("Websocket postinit")
         await self._get_fid()
         await self._get_device_token()
-        await self.refresh_ws_token()
-        self.websocket = WebsocketClient(self)
+
+        # Initialize WebsocketManager ic-dev21
+        self.websocket_manager = WebsocketManager(
+            self.session, self.async_request, self._state_yaml, set_state
+        )
+        await self.websocket_manager.initialize_websockets()
+
+        # Create both websocket clients
+        # ic-dev21 need to work on this as it can't lint as is, may need to
+        # instantiate differently
+        self.websocket_devices = WebsocketClient(self.websocket_manager.devicehub)
+
+        # For backward compatibility during the transition to challengehub websocket
+        self.websocket = self.websocket_devices
+        self.websocket_challenges = WebsocketClient(self.websocket_manager.challengehub)
 
     async def refresh_ws_token(self) -> None:
-        (self.ws_url, self.ws_token) = await self.post_devicehub_negociate()
-        await self.get_websocket_params()
-
-    async def post_devicehub_negociate(self) -> tuple[str, str]:
-        LOG.debug("Getting websocket url")
-        url = f"{AUTOMATION_DEVICEHUB_ENDPOINT}/negotiate"
-        resp = await self.async_request("post", url)
-        ws_url = resp.get("url")
-        ws_token = resp.get("accessToken")
-        LOG.debug("Calling set_state")
-        await set_state(
-            self._state_yaml,
-            "websocket",
-            {
-                "url": ws_url,
-                "token": ws_token,
-            },
-        )
-        return (ws_url, ws_token)
+        """Refresh the websocket token."""
+        await self.websocket_manager.refresh_token(self.websocket_manager.devicehub)
+        await self.websocket_manager.refresh_token(self.websocket_manager.challengehub)
 
     async def get_websocket_params(self) -> None:
         uri = parse.urlparse(self.ws_url)
         LOG.debug("Getting websocket params")
+        LOG.debug(f"Getting uri {uri}")
         resp: dict[str, Any] = await self.async_request(
             "post",
             f"{uri.path}negotiate?{uri.query}",
@@ -391,6 +408,7 @@ class API:
         )
         conn_id: str = resp.get("connectionId", "")
         self.full_ws_url = f"{self.ws_url}&id={conn_id}&access_token={self.ws_token}"
+        LOG.debug(f"Getting full ws URL {self.full_ws_url}")
         transport_dict: list[WebsocketTransportsDict] = resp.get(
             "availableTransports", []
         )
@@ -399,7 +417,7 @@ class API:
             "available_transports": transport_dict,
             "full_ws_url": self.full_ws_url,
         }
-        LOG.debug("Calling set_state")
+        LOG.debug("Calling set_state websocket_params")
         await set_state(self._state_yaml, "websocket", websocket_dict)
 
     async def fb_install(self, fb_id: str) -> None:
@@ -425,7 +443,7 @@ class API:
             raise RequestError(err) from err
         LOG.debug(f"FB Install data: {resp}")
         auth_token = resp.get("authToken", {})
-        LOG.debug("Calling set_state")
+        LOG.debug("Calling set_state fb_install")
         await set_state(
             self._state_yaml,
             "firebase",
@@ -467,7 +485,7 @@ class API:
             LOG.error(f"Android registration error: {msg}")
             raise RequestError
         token = msg.split("=")[-1]
-        LOG.debug("Calling set_state")
+        LOG.debug("Calling set_state android_register")
         await set_state(
             self._state_yaml,
             "android",
@@ -478,12 +496,14 @@ class API:
 
     async def get_location_id(self) -> int:
         url = f"{API_AUTOMATION_ENDPOINT}/Locations"
+        LOG.debug(f"LocationId URL is {url}")
         req: list[dict[str, Any]] = await self.async_request("get", url)
         return int(req[0]["id"])
 
     async def get_devices(self, location_id: int) -> list[dict[str, Any]]:
         """Get list of all devices"""
         url = self._get_url("Devices", location_id)
+        LOG.debug(f"Devices URL is {url}")
         devices: list[dict[str, Any]] = await self.async_request("get", url)
         devices.append(await self.get_gateway(location_id))
         # Now it's time to add devices coming from external sources like hass
@@ -499,6 +519,7 @@ class API:
         value: Union[str, float, int, None],
     ) -> None:
         url = self._get_url(f"Devices/{device.id}/Attributes", device.location_id)
+        LOG.debug(f"Device Attribute URL is {url}")
         await self.async_request("put", url, json={key.hilo_attribute: value})
 
     async def get_event_notifications(self, location_id: int) -> dict[str, Any]:
@@ -526,6 +547,7 @@ class API:
           "viewed": false
         }"""
         url = self._get_url(None, location_id, events=True)
+        LOG.debug(f"Event Notifications URL is {url}")
         return cast(dict[str, Any], await self.async_request("get", url))
 
     async def get_gd_events(
@@ -597,6 +619,8 @@ class API:
             url += "?active=true"
         else:
             url += f"/{event_id}"
+
+        LOG.debug(f"get_gd_events URL is {url}")
         return cast(dict[str, Any], await self.async_request("get", url))
 
     async def get_seasons(self, location_id: int) -> dict[str, Any]:
@@ -624,6 +648,7 @@ class API:
 
     async def get_gateway(self, location_id: int) -> dict[str, Any]:
         url = self._get_url("Gateways/Info", location_id)
+        LOG.debug(f"Gateway URL is {url}")
         req = await self.async_request("get", url)
         saved_attrs = [
             "zigBeePairingActivated",
